@@ -1,4 +1,5 @@
 #!/bin/sh
+# A Kino export script that plays video using smilutils' rawplay
 
 usage()
 {
@@ -13,9 +14,11 @@ usage()
 	echo Flags: single-pass 
 
 	# Profiles
+	echo Profile: Full Screen, ALSA audio output
 	echo Profile: Full Screen, OSS audio output
 	echo Profile: Full Screen, ARTS audio output
 	echo Profile: Full Screen, ESD audio output
+	echo Profile: Windowed, ALSA audio output
 	echo Profile: Windowed, OSS audio output
 	echo Profile: Windowed, ARTS audio output
 	echo Profile: Windowed, ESD audio output
@@ -30,12 +33,14 @@ execute()
 
 	# Run the command
 	case "$profile" in 
-		"0" ) 	rawplay -b -f -ao dsp ;;
-		"1" ) 	rawplay -b -f -ao artsc ;;
-		"2" ) 	rawplay -b -f -ao esd ;;
-		"3" ) 	rawplay -b -ao dsp ;;
-		"4" ) 	rawplay -b -ao artsc ;;
-		"5" ) 	rawplay -b -ao esd ;;
+		"0" ) 	rawplay -b -f -ao alsa ;;
+		"1" ) 	rawplay -b -f -ao dsp ;;
+		"2" ) 	rawplay -b -f -ao artsc ;;
+		"3" ) 	rawplay -b -f -ao esd ;;
+		"4" ) 	rawplay -b -ao alsa ;;
+		"5" ) 	rawplay -b -ao dsp ;;
+		"6" ) 	rawplay -b -ao artsc ;;
+		"7" ) 	rawplay -b -ao esd ;;
 	esac
 }
 
