@@ -1,6 +1,6 @@
 /*
-*
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #endif
 
 #include <iostream>
-using std::cout;
+using std::cerr;
 using std::endl;
 
 // GUI Page Handling Code
@@ -32,7 +32,6 @@ using std::endl;
 #include "page_capture.h"
 #include "page_timeline.h"
 #include "page_export.h"
-
 #include "preferences.h"
 #include "avi.h"
 #include "filehandler.h"
@@ -84,7 +83,9 @@ extern "C"
 			};
 			g_strfreev( urls );
 			windowMoved();
+			gtk_drag_finish( context, TRUE, FALSE, time );
 		}
+		gtk_drag_finish( context, FALSE, FALSE, time );
 	}
 
 	void kinoInitialise( GtkWidget * widget )
@@ -110,6 +111,12 @@ extern "C"
 		                   ( GtkDestDefaults ) ( GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
 		                                         GTK_DEST_DEFAULT_DROP ), file_targets, 1,
 		                   ( GdkDragAction ) ( GDK_ACTION_COPY | GDK_ACTION_MOVE ) );
+		g_signal_connect( G_OBJECT( lookup_widget( widget, "main_notebook" ) ), "drag_data_received",
+		                  G_CALLBACK( getDroppedFiles ), NULL );
+		gtk_drag_dest_set( GTK_WIDGET( lookup_widget( widget, "main_notebook" ) ),
+		                   ( GtkDestDefaults ) ( GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
+		                                         GTK_DEST_DEFAULT_DROP ), file_targets, 1,
+		                   ( GdkDragAction ) ( GDK_ACTION_COPY | GDK_ACTION_MOVE ) );
 		gtk_window_set_default_size( GTK_WINDOW( widget ),
 			Preferences::getInstance().windowWidth, Preferences::getInstance().windowHeight );
 		gtk_paned_set_position( GTK_PANED( lookup_widget( widget , "hpaned1" ) ),
@@ -126,12 +133,13 @@ extern "C"
 			widget2 = lookup_widget( widget, "publish_still" );
 			gtk_container_remove( GTK_CONTAINER( widget2->parent ), widget2 );
 		}
+		common->updateRecentFiles();
 	}
 
 	void kinoPostInit()
 	{
 		common->setPreviewSize( float( Preferences::getInstance().previewSize ) / 10.0, true );
-		notebookChangePage( PAGE_EDITOR );
+		common->setCurrentPage( PAGE_EDITOR );
 	}
 
 	gboolean kinoDeactivate()
@@ -151,7 +159,18 @@ extern "C"
 
 	void bulkLoad( int argc, char * argv[] )
 	{
-		common->bulkLoad( argc, argv );
+		if ( GetEditorBackup()->Restore( common->getPlayList() ) )
+		{
+			common->g_currentFrame = 0;
+			common->hasListChanged = TRUE;
+			common->setWindowTitle();
+			modal_message( _( "Kino has recovered from a crash and restored your undo history!" ) );
+		}
+		else
+		{
+			common->newFile();
+			common->bulkLoad( argc, argv );
+		}
 	}
 
 	void newFile( )
@@ -401,10 +420,11 @@ extern "C"
 	
 	void showHelp( const char *page )
 	{
+		extern char* g_help_language;
 		std::string cmd = "\"" DATADIR "/kino/scripts/help.sh\" \"file://" DATADIR "/kino/help/";
 		if ( page == NULL || strcmp( page, "" ) == 0 )
 			page = common->getCurrentPage()->getHelpPage().c_str();
-		cmd += std::string( page ) + "\" &";
+		cmd += std::string( g_help_language ) + std::string( "\" \"" ) + std::string( page ) + "\" &";
 		system( cmd.c_str() );
 	}
 	
@@ -469,4 +489,115 @@ extern "C"
 		}
 	}
 
+	int kino2raw( char* filename, char* option )
+	{
+		try
+		{
+			PlayList playlist;
+			Frame frame;
+			AudioInfo info;
+	
+			if ( playlist.LoadPlayList( filename ) )
+			{
+				playlist.GetFrame( 0, frame );
+
+				if ( option )
+				{
+					if ( strcmp( option, "aspect" ) == 0 )
+					{
+						std::cout << ( frame.IsWide() ? "16:9" : "4:3" ) << std::endl;
+					}
+					else if ( strcmp( option, "normalisation" ) == 0 )
+					{
+						std::cout << ( frame.IsPAL() ? "pal" : "ntsc" ) << std::endl;
+					}
+					return 0;
+				}
+
+				frame.GetAudioInfo( info );
+				const double outputRate = info.frequency;
+
+				// Determine correct amount of audio for duration
+				double adjustedRate = 0.0;
+				{
+					double seconds = double( playlist.GetNumFrames() ) / frame.GetFrameRate();
+					int64_t idealSamples = int64_t( seconds * outputRate + 0.5 );
+					int64_t actualSamples = 0;
+					int n = -1, frameNum = 0, previousProgress = -1;
+					AsyncAudioResample<int16_ne_t,int16_le_t> resampler(
+						AUDIO_RESAMPLE_SRC_SINC_FASTEST, &playlist, outputRate, 0, playlist.GetNumFrames() - 1, 1 );
+						char buf[ 512 ];
+
+					// Determine the actual number of samples
+					while ( n != 0 )
+					{
+						n = resampler.Process( outputRate, frame.CalculateNumberSamples( int( outputRate ), frameNum++ ) );
+						actualSamples += n;
+						int progress = int( double( frameNum ) / playlist.GetNumFrames() * 100 );
+						if ( progress != previousProgress )
+						{
+							previousProgress = progress;
+							snprintf( buf, 512, "\r%s (%02d%%)", _("Locking audio"), progress );
+							std::cerr << buf;
+						}
+					}
+					adjustedRate = outputRate + ( idealSamples - actualSamples ) / seconds;
+					std::cerr << std::endl;
+				}
+
+				// Set up resampling
+				int16_le_t *audio_buffers[ 4 ];
+				for ( int c = 0; c < 4; c++ )
+					audio_buffers[ c ] = new int16_le_t[ 2 * DV_AUDIO_MAX_SAMPLES ];
+		
+				// Create the resampler
+				AsyncAudioResample<int16_ne_t,int16_le_t> resampler(
+					AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, &playlist, adjustedRate, 0, playlist.GetNumFrames() - 1, 1 );
+				if ( resampler.IsError() )
+				{
+					std::cerr << "Resampler error: " << resampler.GetError() << std::endl;
+					for ( int c = 0; c < 4; c++ )
+						delete[] audio_buffers[ c ];
+					return 4;
+				}
+		
+				for ( int i = 0; i < playlist.GetNumFrames(); ++i )
+				{
+					// Resample
+					int requestedSamples = frame.CalculateNumberSamples( int( outputRate ), i );
+
+					playlist.GetFrame( i, frame );
+					info.samples = resampler.Process( adjustedRate, requestedSamples );
+					if ( info.samples )
+					{
+						// Deinterleave samples
+						int16_le_t *p = resampler.GetOutput();
+						for ( int s = 0; s < info.samples; s++ )
+							for ( int c = 0; c < info.channels; c++ )
+								audio_buffers[ c ][ s ] = *p++;
+
+						frame.EncodeAudio( info, audio_buffers );
+					}
+					if ( !fwrite( frame.data, frame.GetFrameSize(), 1, stdout ) )
+					{
+						perror( "Failed to write raw DV" );
+						for ( int c = 0; c < 4; c++ )
+							delete[] audio_buffers[ c ];
+						return 2;
+					}
+				}
+
+				// Cleanup
+				for ( int c = 0; c < 4; c++ )
+					delete[] audio_buffers[ c ];
+			}
+		}
+		catch (...)
+		{
+			std::cerr << "General Error" << std::endl;
+			return 3;
+		}
+		return 0;
+	}
+
 } // extern "C"
