@@ -1,6 +1,7 @@
 /*
 * export.h Export Mode Base Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2002 Mads Bondo Dydensborg <madsdyd@challenge.dk>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -116,16 +117,17 @@ protected:
 	    Parameters marked with * should really be the ones passed to doExport
 	    (Perhaps these should really be stored in this object... hmmm 
 	    
-	    \param currentFrame the frame to be exported (usually loop parameter)
-	           this should be in playList counting - ie, 1 will be added before
-	    the user sees it.
+	    \param progress the number of frames processed thus far
 	    \param begin first frame to be exported (*)
 	    \param end last frame to be exported (*)
 	    \param every step for frames to be exported
+	    \param currentFrame the frame to be exported (usually loop parameter)
+	           this should be in playList counting - ie, 1 will be added before
+	           the user sees it. -1 (default) = use progress value.
 	    \param activity a short description of what is happening
 	*/
-	void innerLoopUpdate( int currentFrame, int begin, int end, int every,
-		const char* activity = _("Exporting") );
+	void innerLoopUpdate( int progress, int begin, int end, int every,
+		int currentFrame = -1, const char* activity = _("Exporting") );
 
 	/** Determine a new audio sampling rate that will try to maximize the proper
 	    number of audio samples for the duration because DV often contains
