@@ -1,6 +1,7 @@
 /*
 * frame.cc -- utilities for processing digital video frames
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -59,22 +60,29 @@ VideoInfo::VideoInfo() : width( 0 ), height( 0 ), isPAL( false )
 {}
 
 #ifndef HAVE_LIBDV
-bool Frame::maps_initialized = false;
-int Frame::palmap_ch1[ 2000 ];
-int Frame::palmap_ch2[ 2000 ];
-int Frame::palmap_2ch1[ 2000 ];
-int Frame::palmap_2ch2[ 2000 ];
-
-int Frame::ntscmap_ch1[ 2000 ];
-int Frame::ntscmap_ch2[ 2000 ];
-int Frame::ntscmap_2ch1[ 2000 ];
-int Frame::ntscmap_2ch2[ 2000 ];
-
-short Frame::compmap[ 4096 ];
+static bool Frame::maps_initialized = false;
+static int Frame::palmap_ch1[ 2000 ];
+static int Frame::palmap_ch2[ 2000 ];
+static int Frame::palmap_2ch1[ 2000 ];
+static int Frame::palmap_2ch2[ 2000 ];
+static int Frame::ntscmap_ch1[ 2000 ];
+static int Frame::ntscmap_ch2[ 2000 ];
+static int Frame::ntscmap_2ch1[ 2000 ];
+static int Frame::ntscmap_2ch2[ 2000 ];
+static short Frame::compmap[ 4096 ];
 #endif
 
-pthread_mutex_t avcodec_mutex = PTHREAD_MUTEX_INITIALIZER;
-
+// Static globals for encoder, not accessed by more than one thread
+static pthread_mutex_t avcodec_mutex = PTHREAD_MUTEX_INITIALIZER;
+#if defined(HAVE_LIBAVCODEC)
+static AVFormatContext *avformatEncoder = NULL;
+static AVPacket avpacketEncoder;
+static bool isEncoderHeaderWritten = false;
+#if defined(HAVE_SWSCALE)
+static struct SwsContext *imgConvertEncoderCtx = NULL;
+#endif
+#endif
+static uint8_t *tempImage = NULL;
 
 /** constructor
  
@@ -83,17 +91,20 @@ pthread_mutex_t avcodec_mutex = PTHREAD_MUTEX_INITIALIZER;
  
 */
 
-Frame::Frame() : bytesInFrame( 0 ), tempImage( 0 )
+Frame::Frame() : bytesInFrame( 0 )
 {
 #if defined(HAVE_LIBAVCODEC)
 	pthread_mutex_lock( &avcodec_mutex );
 	av_register_all();
 	libavcodec = avcodec_alloc_context();
 	libavcodec->thread_count = 2;
-	avcodec_open( libavcodec, &dvvideo_decoder );
+	avcodec_open( libavcodec, avcodec_find_decoder( CODEC_ID_DVVIDEO ) );
 	pthread_mutex_unlock( &avcodec_mutex );
-	avcodecEncoder = NULL;
 	data = ( unsigned char* ) av_mallocz( 144000 );
+#if defined(HAVE_SWSCALE)
+	imgConvertRgbCtx = NULL;
+	imgConvertYuvCtx = NULL;
+#endif
 #else
 	data = ( unsigned char* ) calloc( 1, 144000 );
 #endif
@@ -174,19 +185,25 @@ Frame::~Frame()
 	pthread_mutex_lock( &avcodec_mutex );
 	avcodec_close( libavcodec );
 	av_free( libavcodec );
-	if ( avcodecEncoder )
+/*	if ( avformatEncoder )
 	{
-		avcodec_close( avcodecEncoder );
-		av_free( avcodecEncoder );
-	}
+		av_write_trailer( avformatEncoder );
+		av_destruct_packet( &avpacketEncoder );
+		avcodec_close( avformatEncoder->streams[0]->codec );
+		av_free( avformatEncoder->streams[0]->codec );
+		av_free( avformatEncoder->streams[0] );
+		av_free( avformatEncoder );
+	}*/
 	pthread_mutex_unlock( &avcodec_mutex );
 	av_free( data );
-	if ( tempImage )
-		av_free( tempImage );
+#if defined(HAVE_SWSCALE)
+	if ( imgConvertRgbCtx )
+		sws_freeContext( imgConvertRgbCtx );
+	if ( imgConvertYuvCtx )
+		sws_freeContext( imgConvertYuvCtx );
+#endif
 #else
 	free( data );
-	if ( tempImage )
-		delete[] tempImage;
 #endif
 
 #ifdef HAVE_LIBDV
@@ -775,11 +792,16 @@ int Frame::ExtractAudio( void *sound ) const
 	{
 		int n, i;
 		int16_t* s = ( int16_t * ) sound;
-		dv_decode_full_audio( decoder, data, ( int16_t ** ) audio_buffers );
-		for ( n = 0; n < info.samples; ++n )
-			for ( i = 0; i < info.channels; i++ )
-				*s++ = audio_buffers[ i ][ n ];
-
+		if ( dv_decode_full_audio( decoder, data, ( int16_t ** ) audio_buffers ) == TRUE )
+		{
+			for ( n = 0; n < info.samples; ++n )
+				for ( i = 0; i < info.channels; i++ )
+					*s++ = audio_buffers[ i ][ n ];
+		}
+		else
+		{
+			info.samples = 0;
+		}
 	}
 	else
 		info.samples = 0;
@@ -856,58 +878,6 @@ int Frame::ExtractAudio( void *sound ) const
 	return info.samples * info.channels * 2;
 }
 
-#ifdef HAVE_LIBDV
-
-void Frame::SetPreferredQuality( )
-{
-	switch ( Preferences::getInstance().displayQuality )
-	{
-	case 5:
-		dv_set_quality( decoder, DV_QUALITY_FASTEST );
-		break;
-	case 4:
-		dv_set_quality( decoder, DV_QUALITY_AC_1 );
-		break;
-	case 3:
-		dv_set_quality( decoder, DV_QUALITY_COLOR | DV_QUALITY_DC );
-		break;
-	case 2:
-		dv_set_quality( decoder, DV_QUALITY_COLOR | DV_QUALITY_AC_1 );
-		break;
-	default:
-		dv_set_quality( decoder, DV_QUALITY_BEST );
-		break;
-	}
-}
-
-/** retrieves the audio data from the frame
- 
-    The DV frame contains audio data mixed in the video data blocks, 
-    which can be retrieved easily using this function.
- 
-    The audio data consists of 16 bit, two channel audio samples (a 16 bit word for channel 1, followed by a 16 bit word
-    for channel 2 etc.)
- 
-    \param channels an array of buffers of audio data, one per channel, up to four channels
-    \return the number of bytes put into the buffer, or 0 if no audio data could be retrieved */
-
-int Frame::ExtractAudio( int16_t **channels ) const
-{
-	AudioInfo info;
-
-	if ( GetAudioInfo( info ) == true )
-		dv_decode_full_audio( decoder, data, channels );
-	else
-		info.samples = 0;
-
-	return info.samples * info.channels * 2;
-}
-
-void Frame::ExtractHeader( void )
-{
-	dv_parse_header( decoder, data );
-	dv_parse_packs( decoder, data );
-}
 
 void Frame::GetUpperField( void * image, int bpp )
 {
@@ -917,6 +887,7 @@ void Frame::GetUpperField( void * image, int bpp )
 		memcpy( ( uint8_t * ) image + width * ( i + 1 ), ( uint8_t * ) image + width * i, width );
 }
 
+
 void Frame::GetLowerField( void * image, int bpp )
 {
 	register int width = GetWidth( ) * bpp;
@@ -925,6 +896,7 @@ void Frame::GetLowerField( void * image, int bpp )
 		memcpy( ( uint8_t * ) image + width * i, ( uint8_t * ) image + width * ( i + 1 ), width );
 }
 
+
 /* Linear Blend filter - C version contributed by Rogerio Brito.
    This algorithm has the same interface as the other functions.
 
@@ -973,6 +945,103 @@ void Frame::Deinterlace( uint8_t *pdst, uint8_t *psrc, int stride, int height )
 }
 
 
+/** Get the frame aspect ratio.
+ 
+	Indicates whether frame aspect ration is normal (4:3) or wide (16:9).
+ 
+    \return true if the frame is wide (16:9), false if unknown or normal.
+*/
+bool Frame::IsWide( void ) const
+{
+	// Not using libdv due to libdv bug setting widescreen properly when encoding metadata
+	// on SMPTE DV as encoded by ffmpeg.
+	// return dv_format_wide( decoder ) > 0;
+	Pack pack;
+	if ( GetVAUXPack( 0x61, pack ) )
+		return ( ( pack.data[2] & 0x7 ) == 0x2 ) || ( ( pack.data[2] & 0x7 ) == 0x7 );
+	else
+		return false;
+}
+
+
+/** Get the frame image width.
+ 
+    \return the width in pixels.
+*/
+int Frame::GetWidth()
+{
+	return 720;
+}
+
+
+/** Get the frame image height.
+ 
+    \return the height in pixels.
+*/
+int Frame::GetHeight()
+{
+	return IsPAL() ? 576 : 480;
+}
+
+
+#ifdef HAVE_LIBDV
+
+void Frame::SetPreferredQuality( )
+{
+	switch ( Preferences::getInstance().displayQuality )
+	{
+	case 5:
+		dv_set_quality( decoder, DV_QUALITY_FASTEST );
+		break;
+	case 4:
+		dv_set_quality( decoder, DV_QUALITY_AC_1 );
+		break;
+	case 3:
+		dv_set_quality( decoder, DV_QUALITY_COLOR | DV_QUALITY_DC );
+		break;
+	case 2:
+		dv_set_quality( decoder, DV_QUALITY_COLOR | DV_QUALITY_AC_1 );
+		break;
+	default:
+		dv_set_quality( decoder, DV_QUALITY_BEST );
+		break;
+	}
+}
+
+/** retrieves the audio data from the frame
+ 
+    The DV frame contains audio data mixed in the video data blocks, 
+    which can be retrieved easily using this function.
+ 
+    The audio data consists of 16 bit, two channel audio samples (a 16 bit word for channel 1, followed by a 16 bit word
+    for channel 2 etc.)
+ 
+    \param channels an array of buffers of audio data, one per channel, up to four channels
+    \return the number of bytes put into the buffer, or 0 if no audio data could be retrieved */
+
+int Frame::ExtractAudio( int16_t **channels ) const
+{
+	AudioInfo info;
+
+	if ( GetAudioInfo( info ) )
+	{
+		if ( dv_decode_full_audio( decoder, data, channels ) == FALSE )
+			info.samples = 0;
+	}
+	else
+	{
+		info.samples = 0;
+	}
+
+	return info.samples * info.channels * 2;
+}
+
+void Frame::ExtractHeader( void )
+{
+	dv_parse_header( decoder, data );
+	dv_parse_packs( decoder, data );
+}
+
 int Frame::ExtractRGB( void * rgb )
 {
 #if defined(HAVE_LIBAVCODEC)
@@ -984,7 +1053,14 @@ int Frame::ExtractRGB( void * rgb )
 	if ( got_picture )
 	{
 		avpicture_fill( &dest, static_cast<uint8_t*>( rgb ), PIX_FMT_RGB24, GetWidth(), GetHeight() );
+#if defined(HAVE_SWSCALE)
+		if ( !imgConvertRgbCtx )
+			imgConvertRgbCtx = sws_getContext( libavcodec->width, libavcodec->height, libavcodec->pix_fmt,
+				GetWidth(), GetHeight(), PIX_FMT_RGB24, SWS_FAST_BILINEAR, NULL, NULL, NULL );
+		sws_scale( imgConvertRgbCtx, frame->data, frame->linesize, 0, libavcodec->height, dest.data, dest.linesize );
+#else
 		img_convert( &dest, PIX_FMT_RGB24, reinterpret_cast<AVPicture*>( frame ), libavcodec->pix_fmt, GetWidth(), GetHeight() );
+#endif
 	}
 	av_free( frame );
 #else
@@ -1037,7 +1113,14 @@ int Frame::ExtractYUV( void *yuv )
 	if ( got_picture )
 	{
 		avpicture_fill( &output, static_cast<uint8_t*>( yuv ), PIX_FMT_YUV422, GetWidth(), GetHeight() );
+#if defined(HAVE_SWSCALE)
+		if ( !imgConvertYuvCtx )
+			imgConvertYuvCtx = sws_getContext( libavcodec->width, libavcodec->height, libavcodec->pix_fmt,
+				GetWidth(), GetHeight(), PIX_FMT_YUV422, SWS_FAST_BILINEAR, NULL, NULL, NULL );
+		sws_scale( imgConvertYuvCtx, frame->data, frame->linesize, 0, libavcodec->height, output.data, output.linesize );
+#else
 		img_convert( &output, PIX_FMT_YUV422, (AVPicture *)frame, libavcodec->pix_fmt, GetWidth(), GetHeight() );
+#endif
 	}
 	av_free( frame );
 #else
@@ -1184,38 +1267,6 @@ int Frame::ExtractPreviewYUV( void *yuv )
 }
 
 
-/** Get the frame aspect ratio.
- 
-	Indicates whether frame aspect ration is normal (4:3) or wide (16:9).
- 
-    \return true if the frame is wide (16:9), false if unknown or normal.
-*/
-bool Frame::IsWide( void ) const
-{
-	return dv_format_wide( decoder ) > 0;
-}
-
-
-/** Get the frame image width.
- 
-    \return the width in pixels.
-*/
-int Frame::GetWidth()
-{
-	return 720;
-}
-
-
-/** Get the frame image height.
- 
-    \return the height in pixels.
-*/
-int Frame::GetHeight()
-{
-	return IsPAL() ? 576 : 480;
-}
-
-
 /** Set the RecordingDate of the frame.
  
     This updates the calendar date and time and the timecode.
@@ -1250,29 +1301,49 @@ void Frame::SetTimeCode( int frame )
 
 bool Frame::CreateEncoder( bool isPAL, bool isWide )
 {
+	pthread_mutex_lock( &avcodec_mutex );
 #if defined(HAVE_LIBAVCODEC)
-	if ( avcodecEncoder == NULL )
+	if ( avformatEncoder == NULL )
 	{
-		avcodecEncoder = avcodec_alloc_context();
-		if ( avcodecEncoder )
+		avformatEncoder = av_alloc_format_context();
+		if ( avformatEncoder )
 		{
-			pthread_mutex_lock( &avcodec_mutex );
-			avcodec_open( avcodecEncoder, &dvvideo_encoder );
-			pthread_mutex_unlock( &avcodec_mutex );
-			if ( isPAL )
- 				avcodecEncoder->sample_aspect_ratio = isWide ? ( AVRational ){ 118, 81 } : ( AVRational ){ 59, 54 };
-			else
- 				avcodecEncoder->sample_aspect_ratio = isWide ? ( AVRational ){ 40, 33 } : ( AVRational ){ 10, 11 };
-			avcodecEncoder->thread_count = 2;
+			avformatEncoder->oformat = guess_format( "dv", NULL, NULL );
+			av_new_stream( avformatEncoder, 0 );
+			AVCodecContext *avcodecEncoder = avformatEncoder->streams[0]->codec;
 			avcodecEncoder->width = FRAME_MAX_WIDTH;
 			avcodecEncoder->height = isPAL ? 576 : 480;
+			if ( isPAL )
+			{
+				if ( isWide )
+					avcodecEncoder->sample_aspect_ratio = av_d2q( 17.0 / avcodecEncoder->width * avcodecEncoder->height / 10.0, 255 );
+				else
+ 					avcodecEncoder->sample_aspect_ratio = isWide ? ( AVRational ){ 118, 81 } : ( AVRational ){ 59, 54 };
+			}
+			else
+			{
+				if ( isWide )
+					avcodecEncoder->sample_aspect_ratio = av_d2q( 17.0 / avcodecEncoder->width * avcodecEncoder->height / 10.0, 255 );
+				else
+ 					avcodecEncoder->sample_aspect_ratio = isWide ? ( AVRational ){ 40, 33 } : ( AVRational ){ 10, 11 };
+			}
+			avcodecEncoder->thread_count = 2;
 			avcodecEncoder->time_base= isPAL ? ( AVRational ){ 1, 25 } : ( AVRational ){ 1001, 30000 };
 			avcodecEncoder->pix_fmt = isPAL ? PIX_FMT_YUV420P : PIX_FMT_YUV411P;
-			tempImage = ( uint8_t* ) av_malloc( 
+			avcodecEncoder->flags |= CODEC_FLAG_INTERLACED_DCT;
+			av_set_parameters( avformatEncoder, NULL );
+			avcodec_open( avcodecEncoder, avcodec_find_encoder( CODEC_ID_DVVIDEO ) );
+			av_new_packet( &avpacketEncoder, 144000 );
+			tempImage = ( uint8_t* ) av_malloc(
 				avpicture_get_size( avcodecEncoder->pix_fmt, avcodecEncoder->width, avcodecEncoder->height ) );
+#if defined(HAVE_SWSCALE)
+			imgConvertEncoderCtx = sws_getContext( avcodecEncoder->width, avcodecEncoder->height, PIX_FMT_RGB24,
+				avcodecEncoder->width, avcodecEncoder->height, avcodecEncoder->pix_fmt, SWS_FAST_BILINEAR, NULL, NULL, NULL);
+#endif
 		}
 		else
 		{
+			pthread_mutex_unlock( &avcodec_mutex );
 			return false;
 		}
 	}
@@ -1296,21 +1367,40 @@ bool Frame::CreateEncoder( bool isPAL, bool isWide )
 			tempImage = new uint8_t[ FRAME_MAX_WIDTH * FRAME_MAX_HEIGHT * 4 ];
 		}
 	}
+	pthread_mutex_unlock( &avcodec_mutex );
 	return ( encoder != NULL );
 }
 
-bool Frame::EncodeAudio( AudioInfo &info, int16_t **channels )
+bool Frame::EncodeAudio( AudioInfo &info, int16_le_t **channels )
 {
 	int result = -1;
 
 	if ( CreateEncoder( IsPAL(), IsWide() ) )
 	{
 		encoder->samples_this_frame = info.samples;
-		result = dv_encode_full_audio( encoder, channels, info.channels, info.frequency, data );
+		result = dv_encode_full_audio( encoder, (int16_t**)channels, info.channels, info.frequency, data );
 	}
 	return ( result != -1 );
 }
 
+#if BYTE_ORDER == BIG_ENDIAN
+bool Frame::EncodeAudio( AudioInfo &info, int16_ne_t **channels )
+{
+	int result = -1;
+
+	if ( CreateEncoder( IsPAL(), IsWide() ) )
+	{
+		int16_le_t** le_audio = reinterpret_cast< int16_le_t** >( audio_buffers );
+		for ( int s = 0; s < info.samples; s++ )
+			for ( int c = 0; c < info.channels; c++ )
+				le_audio[ c ][ s ] = channels[ c ][ s ];
+		encoder->samples_this_frame = info.samples;
+		result = dv_encode_full_audio( encoder, audio_buffers, info.channels, info.frequency, data );
+	}
+	return ( result != -1 );
+}
+#endif
+
 int Frame::CalculateNumberSamples( int frequency, int iteration )
 {
 	int samples = 0;
@@ -1331,6 +1421,7 @@ void Frame::EncodeRGB( uint8_t *rgb )
 
 		if ( input && output )
 		{
+			AVCodecContext *avcodecEncoder = avformatEncoder->streams[0]->codec;
 			int width = avcodecEncoder->width;
 			int height = avcodecEncoder->height;
 			size_t size = height > 480 ? 144000 : 120000;
@@ -1338,37 +1429,31 @@ void Frame::EncodeRGB( uint8_t *rgb )
 			// Convert color space
 			avpicture_fill( ( AVPicture* )output, tempImage, avcodecEncoder->pix_fmt, width, height );
 			avpicture_fill( ( AVPicture* )input, rgb, PIX_FMT_RGB24, width, height );
+#if defined(HAVE_SWSCALE)
+			sws_scale( imgConvertEncoderCtx, input->data, input->linesize, 0, height,
+				output->data, output->linesize);
+#else
 			img_convert( ( AVPicture* )output, avcodecEncoder->pix_fmt, ( AVPicture* )input, PIX_FMT_RGB24, width, height );
+#endif
 
 			// Encode
-			bytesInFrame = avcodec_encode_video( avcodecEncoder, data, size, output );
-
-			// Use libavformat to mux the DV  because libavcodec produces an incomplete DIF bytstream
-			AVFormatContext *oc = av_alloc_format_context();
-			oc->oformat = guess_format( "dv", NULL, NULL );
-			AVStream *video_st = av_new_stream(oc, 0);
-			AVPacket pkt;
-			uint8_t* buf = new uint8_t[ bytesInFrame ];
-		
-			av_set_parameters(oc, NULL);
-			url_open_buf( &oc->pb, buf, bytesInFrame, URL_WRONLY );
-			video_st->codec = avcodecEncoder;
-			av_write_header(oc);
-			av_init_packet(&pkt);
-			pkt.stream_index = video_st->index;
-			pkt.data = data;
-			pkt.size = bytesInFrame;
-			av_write_frame(oc, &pkt);
-			url_close_buf(&oc->pb);
-			memcpy(data, buf, bytesInFrame);
-			av_write_trailer(oc);
+			bytesInFrame = avcodec_encode_video( avcodecEncoder, avpacketEncoder.data, size, output );
+			url_open_buf( &avformatEncoder->pb, data, bytesInFrame, URL_WRONLY );
+			avpacketEncoder.size = bytesInFrame;
+			if ( !isEncoderHeaderWritten )
+			{
+				av_write_header( avformatEncoder );
+				isEncoderHeaderWritten = true;
+			}
+			av_write_frame( avformatEncoder, &avpacketEncoder );
+			url_close_buf( &avformatEncoder->pb );
 
 			// Update this frame's metadata
 			ExtractHeader();
 
-			delete[] buf;
-			av_freep(&video_st);
-			av_free(oc);
+			// Force the output to be IEC 61834
+			data[4] &= 0xf8;
+
 			av_free( output );
 			av_free( input );
 		}
