@@ -1,6 +1,7 @@
 /*
 * kino_av_pipe.cc -- AV Export Pipe Implementations and Factories
 * Copyright (C) 2002 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2002-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,10 +26,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include "error.h"
 #include "kino_av_pipe.h"
 #include "endian_types.h"
+#include "rwpipe.h"
+#include "frame.h"
+#include "stringutils.h"
 
 /*
  * C Functions for generating a WAV output. Should be rewritten in C++ 
@@ -189,7 +196,7 @@ public:
 	{
 		return length / channels / bytespersample;
 	}
-	bool Seek( long offset );
+	bool Seek( off_t offset );
 	bool Get( int16_t *data, int length );
 	bool Close( );
 };
@@ -243,7 +250,7 @@ bool WAVImport::Open( char *input )
 	return iswav;
 }
 
-bool WAVImport::Seek( long offset )
+bool WAVImport::Seek( off_t offset )
 {
 	return fseek( wav.file, offset + start, SEEK_SET ) == 0;
 }
@@ -266,14 +273,108 @@ bool WAVImport::Close( )
 	return true;
 }
 
+
+class PipeImport : public KinoAudioInput
+{
+private:
+	RWPipe m_pipe;
+	string m_file;
+	int16_t m_buffer[ 4 * DV_AUDIO_MAX_SAMPLES ];
+	char *m_command;
+public:
+	PipeImport()
+		: m_command( 0 )
+	{
+	}
+	virtual ~PipeImport( )
+	{
+		m_pipe.stop();
+		delete m_command;
+	}
+	bool Open( char *input )
+	{
+		bool result = false;
+		char *kinoHome = getenv("KINO_HOME");
+		string homeFile, script;
+		int testHomeFile = 0;
+
+		// Allow a script in home directory to override
+		if ( kinoHome )
+			homeFile = string( kinoHome ) + string( "/import/audio.sh" );
+		else
+			homeFile = string( getenv( "HOME" ) ) + string( "/kino/import/audio.sh" );
+		testHomeFile = open( homeFile.c_str(), O_RDONLY );
+		if ( testHomeFile > -1 )
+		{
+			close( testHomeFile );
+			script = homeFile;
+		}
+		else
+		{
+			script = string( DATADIR "/kino/scripts/import/audio.sh" );
+		}
+
+		delete m_command;
+		string inputFilename = StringUtils::replaceAll( input, "\"", "\\\"" );
+		m_command = g_strdup_printf( "%s \"%s\"", script.c_str(), inputFilename.c_str() );
+		m_pipe.stop();
+		if ( m_pipe.run( m_command ) )
+		{
+			char temp[ 10 ];
+			result = ( m_pipe.readData( temp, 10 ) == 10 );
+			m_pipe.stop();
+			if ( result )
+				return m_pipe.run( m_command );
+		}
+		return result;
+	}
+	int GetChannels( )
+	{
+		return 2;
+	}
+	int GetFrequency( )
+	{
+		return 44100;
+	}
+	int GetBytesPerSample( )
+	{
+		return 2;
+	}
+	long GetNumberOfSamples( )
+	{
+		return -1;
+	}
+	bool Seek( off_t offset )
+	{
+		return false;
+	}
+	bool Get( int16_t *data, int length )
+	{
+		return ( m_pipe.readData( data, length ) == length );
+	}
+	bool Close( )
+	{
+		m_pipe.stop();
+		return true;
+	}
+};
+
+
 KinoAudioInput *KinoAudioInputFactory::CreateAudioInput( char *input )
 {
-	KinoAudioInput * wav = new WAVImport( );
-	if ( wav->Open( input ) )
-		return wav;
+	const std::string filename( input );
+	const std::string suffix( filename.begin() + filename.rfind( "." ), filename.end() );
+	KinoAudioInput* filter;
+
+	if ( suffix == ".wav" )
+		filter = new WAVImport();
 	else
-		delete wav;
-	return NULL;
+		filter = new PipeImport();
+	if ( filter->Open( input ) )
+		return filter;
+	else
+		delete filter;
+	return 0;
 }
 
 /** WAV Export Pipe.
@@ -520,7 +621,6 @@ bool KinoMJPEGVideoPipe::OpenVideoPipe( char *pipe, int width, int height, bool
 
 	int written = 0;
 	if ( output != NULL )
-//		written = fprintf( output, "YUV4MPEG2 W%d H%d F%s\n", width, height, height == 576 ? "25:1 Ib A59:54" : "30000:1001 Ib A10:11" );
 		written = fprintf( output, "YUV4MPEG2 W%d H%d F%s %s\n", width, height, 
 			height == 576 ? "25:1 A118:81" : "30000:1001 A40:33", interlaced ? "Ib" : "Ip" );
 
@@ -611,6 +711,10 @@ bool DvEncoderVideoPipeYUV::OpenVideoPipe( char *pipe, int width, int height, bo
 
 	this->width = width;
 	this->height = height;
+	char output[ 256 ];
+	sprintf( output, "YUV4MPEG2 W%d H%d F%s %s XYSCSS=422\n", width, height,
+			height == 576 ? "25:1 A118:81" : "30000:1001 A40:33", interlaced ? "Ib" : "Ip" );
+	write( m_writer, output, strlen( output ) );
 	return m_writer != -1 && sigpipe_get() == 0;
 }
 
@@ -621,7 +725,7 @@ bool DvEncoderVideoPipeYUV::OutputVideoFrame( uint8_t *frame, int size )
 {
 	sigpipe_clear();
 	char output[ 256 ];
-	sprintf( output, "YUV422\n%d %d\n255\n", width, height );
+	sprintf( output, "FRAME\n" );
 	write( m_writer, output, strlen( output ) );
 	write( m_writer, frame, size );
 	return sigpipe_get() == 0;
