@@ -1,6 +1,6 @@
 /*
 * page_export_1394.cc Notebook Firewire/AVI/Still Frame Export Page Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -80,10 +80,9 @@ void Export1394::start()
 			avc = new AVC();
 			Preferences::getInstance( ).phyID = avc->getNodeId( Preferences::getInstance( ).avcGUID );
 			Preferences::getInstance( ).phyID = avc->isPhyIDValid( Preferences::getInstance( ).phyID );
-			system_loaded = Preferences::getInstance( ).phyID >= 0;
-			if ( !system_loaded )
+			system_loaded = true;
+			if ( avc->getPort() < 0 )
 			{
-				common->setStatusBar( _( "No AV/C compliant cam connected or not switched on?" ) );
 				delete avc;
 				avc = 0;
 			}
@@ -98,17 +97,36 @@ void Export1394::start()
 	else
 	{
 		system_loaded = false;
-		common->setStatusBar( _( "The IEEE 1394 subsystem is not loaded; all IEEE 1394 options are disabled." ) );
+		common->setStatusBar( _( "WARNING: raw1394 kernel module not loaded or failure to read/write /dev/raw1394!" ) );
 	}
 #else
 	else
 	{
-		system_loaded = false;
 		common->setStatusBar( _( "Failed to open raw1394; you must manually control recording on the device." ) );
 	}
 #endif
 
 #ifdef HAVE_IEC61883
+	exportWriter = new iec61883Writer( ( avc && avc->getPort() > -1 ) ? avc->getPort() : 0,
+					Preferences::getInstance().channel,
+					Preferences::getInstance().dvExportBuffers );
+#else
+	exportWriter = new dv1394Writer(
+					   Preferences::getInstance().dvExportDevice,
+					   Preferences::getInstance().channel,
+					   Preferences::getInstance().dvExportBuffers,
+					   Preferences::getInstance().cip_n,
+					   Preferences::getInstance().cip_d,
+					   Preferences::getInstance().syt_offset
+				   );
+	system_loaded = exportWriter->isValid();
+	if ( !system_loaded )
+	{
+		common->setStatusBar( _( "WARNING: dv1394 kernel module not loaded or failure to read/write %s." ),
+			 Preferences::getInstance().dvExportDevice );
+	}
+#endif
+
 	/* Change the sensitivity of the hosting widget according to
 	   wheter the system loaded or not */
 	GtkVBox * tmp
@@ -122,24 +140,6 @@ void Export1394::start()
 	{
 		gtk_widget_set_sensitive ( GTK_WIDGET ( tmp ), FALSE );
 	}
-#endif
-
-#ifdef HAVE_IEC61883
-	if ( avc )
-		exportWriter = new iec61883Writer( avc->getPort(),
-					   Preferences::getInstance().channel,
-					   Preferences::getInstance().dvExportBuffers );
-#else
-	exportWriter = new dv1394Writer(
-					   Preferences::getInstance().dvExportDevice,
-					   Preferences::getInstance().channel,
-					   Preferences::getInstance().dvExportBuffers,
-					   Preferences::getInstance().cip_n,
-					   Preferences::getInstance().cip_d,
-					   Preferences::getInstance().syt_offset
-				   );
-#endif
-
 }
 
 /** Define active widgets.
@@ -151,17 +151,12 @@ gulong Export1394::onActivate()
 {
 	if ( system_loaded )
 	{
-		return EXPORT_PREVIEW | EXPORT_EXPORT | EXPORT_SCENE_LIST;
+		if ( avc )
+			return EXPORT_PREVIEW | EXPORT_EXPORT | EXPORT_SCENE_LIST;
+		else
+			return EXPORT_PREVIEW | EXPORT_SCENE_LIST;
 	}
-#ifdef HAVE_IEC61883
-	else
-		return 0;
-#else
-	else
-	{
-		return EXPORT_PREVIEW | EXPORT_SCENE_LIST;
-	}
-#endif
+	return 0;
 }
 
 /** Leaving the page
@@ -184,9 +179,8 @@ Export1394::doExport( PlayList * playlist, int begin, int end, int every,
 {
 	int i = -1;
 	Preferences::getInstance().dvExportPrerollSec = atoi( gtk_entry_get_text( entry_preroll ) );
-	enum export_result result = EXPORT_RESULT_FAILURE;
+	enum export_result result = EXPORT_RESULT_ABORT;
 	string filename = "";
-	string currentFilename = "";
 
 	// Only for export, NOT for preview.
 	if ( !preview && avc )
@@ -202,14 +196,16 @@ Export1394::doExport( PlayList * playlist, int begin, int end, int every,
 		bool resample = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(
 		                    lookup_widget( common->getWidget(), "checkbutton_export_1394_resample" ) ) );
 
-		int16_t *audio_buffers[ 4 ];
+		int16_le_t *audio_buffers[ 4 ];
 		AudioInfo info;
 		AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = 0;
-		double adjustedRate;
+		double adjustedRate = 0;
 		int frameNum = 0;
 
 		for ( i = 0; i < 4; i++ )
-			audio_buffers[ i ] = ( int16_t * ) calloc( 2 * DV_AUDIO_MAX_SAMPLES, sizeof( int16_t ) );
+			audio_buffers[ i ] = ( int16_le_t * ) calloc( 2 * DV_AUDIO_MAX_SAMPLES, sizeof( int16_t ) );
+
+		innerLoopUpdate( begin, begin, end, every );
 
 		// Calculate resample rate for locked audio
 		playlist->GetFrame( begin, frame );
@@ -232,23 +228,17 @@ Export1394::doExport( PlayList * playlist, int begin, int end, int every,
 			frame.GetAudioInfo( info );
 			info.channels = 2;
 			frame.EncodeAudio( info, audio_buffers );
-			exportWriter->SendFrame( frame );
+			if ( !exportWriter->SendFrame( frame ) )
+			{
+				exportPage->isExporting = false;
+				result = EXPORT_RESULT_FAILURE;
+			}
 		}
 
 		/* Iterate over all frames in selection */
 		for ( i = begin; i <= end && exportPage->isExporting; i += every, frameNum++ )
 		{
-			/* Call innerLoopUpdate */
 			innerLoopUpdate( i, begin, end, every );
-
-			/* display filename */
-			currentFilename = playlist->GetFileNameOfFrame( i );
-			if ( currentFilename != filename )
-			{
-				filename = currentFilename;
-				common->setStatusBar( filename.c_str() );
-			}
-
 			playlist->GetFrame( i, frame );
 
 			/* resample audio */
@@ -275,7 +265,7 @@ Export1394::doExport( PlayList * playlist, int begin, int end, int every,
 				}
 				int requestedSamples = frame.CalculateNumberSamples( info.frequency, frameNum );
 				info.samples = resampler->Process( adjustedRate, requestedSamples );
-				int16_t *p = reinterpret_cast<int16_t *>( resampler->GetOutput() );
+				int16_le_t *p = resampler->GetOutput();
 				for ( int s = 0; s < info.samples; s++ )
 					for ( int c = 0; c < info.channels; c++ )
 						audio_buffers[ c ][ s ] = *p++;
@@ -283,12 +273,14 @@ Export1394::doExport( PlayList * playlist, int begin, int end, int every,
 					frame.EncodeAudio( info, audio_buffers );
 			}
 
-			exportWriter->SendFrame( frame );
+			if ( !exportWriter->SendFrame( frame ) )
+			{
+				result = EXPORT_RESULT_FAILURE;
+				break;
+			}
 		}
 
-		if ( !exportPage->isExporting )
-			result = EXPORT_RESULT_ABORT;
-		else if ( i > end )
+		if ( result != EXPORT_RESULT_FAILURE && i > end )
 			result = EXPORT_RESULT_SUCCESS;
 			
 		for ( i = 0; i < 4; i++ )
