@@ -1,7 +1,6 @@
 /*
-* page_edit.h Notebook Trim Page Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
-* Custom GTK+ Trim Control by Ronald Bultje
+* page_trim.h Notebook Trim Page Object
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -169,7 +168,7 @@ public:
 	void timeFormatChanged();
 	std::string getHelpPage()
 	{
-		return "trim.html";
+		return "trim";
 	}
 };
 
