@@ -1,5 +1,6 @@
 /*
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -805,7 +806,8 @@ extern "C"
 	on_button_capture_open_clicked( GtkButton       *button,
 	                                gpointer         user_data )
 	{
-		char *filename = common->getFileToOpen( _("Choose a DV file"), true, GTK_WIDGET(button) );
+		int dummy = 0;
+		char *filename = common->getFileToSaveFormat( _("Choose a DV file"), GTK_WIDGET(button), dummy );
 		if ( filename && strcmp( filename, "" ) )
 		{
 			gtk_entry_set_text( GTK_ENTRY( lookup_widget( GTK_WIDGET( button ), "entry_file" ) ),
